@@ -14,10 +14,26 @@
 
 package cache
 
-type DefaultCache map[string]bool
+import "time"
+
+type cacheItem struct {
+	value     bool
+	expiresAt time.Time
+	ttl       time.Duration
+}
+
+type DefaultCache map[string]cacheItem
 
 func (c *DefaultCache) Set(key string, value bool, extra ...interface{}) error {
-	(*c)[key] = value
+	ttl := time.Duration(-1)
+	if len(extra) > 0 {
+		ttl = extra[0].(time.Duration)
+	}
+	(*c)[key] = cacheItem{
+		value:     value,
+		expiresAt: time.Now().Add(ttl),
+		ttl:       ttl,
+	}
 	return nil
 }
 
@@ -25,7 +41,11 @@ func (c *DefaultCache) Get(key string) (bool, error) {
 	if res, ok := (*c)[key]; !ok {
 		return false, ErrNoSuchKey
 	} else {
-		return res, nil
+		if res.ttl > 0 && time.Now().After(res.expiresAt) {
+			delete(*c, key)
+			return false, ErrNoSuchKey
+		}
+		return res.value, nil
 	}
 }
 
@@ -42,3 +62,8 @@ func (c *DefaultCache) Clear() error {
 	*c = make(DefaultCache)
 	return nil
 }
+
+func NewDefaultCache() (Cache, error) {
+	cache := make(DefaultCache)
+	return &cache, nil
+}
