@@ -15,6 +15,7 @@
 package casbin
 
 import (
+	"fmt"
 	"strings"
 
 	"github.com/casbin/casbin/v2/constant"
@@ -24,13 +25,21 @@ import (
 
 // GetRolesForUser gets the roles that a user has.
 func (e *Enforcer) GetRolesForUser(name string, domain ...string) ([]string, error) {
-	res, err := e.model["g"]["g"].RM.GetRoles(name, domain...)
+	rm := e.GetRoleManager()
+	if rm == nil {
+		return nil, fmt.Errorf("role manager is not initialized")
+	}
+	res, err := rm.GetRoles(name, domain...)
 	return res, err
 }
 
 // GetUsersForRole gets the users that has a role.
 func (e *Enforcer) GetUsersForRole(name string, domain ...string) ([]string, error) {
-	res, err := e.model["g"]["g"].RM.GetUsers(name, domain...)
+	rm := e.GetRoleManager()
+	if rm == nil {
+		return nil, fmt.Errorf("role manager is not initialized")
+	}
+	res, err := rm.GetUsers(name, domain...)
 	return res, err
 }
 
@@ -114,7 +123,12 @@ func (e *Enforcer) DeleteUser(user string) (bool, error) {
 // Returns false if the role does not exist (aka not affected).
 func (e *Enforcer) DeleteRole(role string) (bool, error) {
 	var err error
-	res1, err := e.RemoveFilteredGroupingPolicy(1, role)
+	res1, err := e.RemoveFilteredGroupingPolicy(0, role)
+	if err != nil {
+		return res1, err
+	}
+
+	res2, err := e.RemoveFilteredGroupingPolicy(1, role)
 	if err != nil {
 		return res1, err
 	}
@@ -123,8 +137,8 @@ func (e *Enforcer) DeleteRole(role string) (bool, error) {
 	if err != nil {
 		return false, err
 	}
-	res2, err := e.RemoveFilteredPolicy(subIndex, role)
-	return res1 || res2, err
+	res3, err := e.RemoveFilteredPolicy(subIndex, role)
+	return res1 || res2 || res3, err
 }
 
 // DeletePermission deletes a permission.
@@ -185,20 +199,24 @@ func (e *Enforcer) GetNamedPermissionsForUser(ptype string, user string, domain
 		args[subIndex] = user
 
 		if len(domain) > 0 {
-			index, err := e.GetFieldIndex(ptype, constant.DomainIndex)
+			var index int
+			index, err = e.GetFieldIndex(ptype, constant.DomainIndex)
 			if err != nil {
 				return permission, err
 			}
 			args[index] = domain[0]
 		}
-		perm := e.GetFilteredNamedPolicy(ptype, 0, args...)
+		perm, err := e.GetFilteredNamedPolicy(ptype, 0, args...)
+		if err != nil {
+			return permission, err
+		}
 		permission = append(permission, perm...)
 	}
 	return permission, nil
 }
 
 // HasPermissionForUser determines whether a user has a permission.
-func (e *Enforcer) HasPermissionForUser(user string, permission ...string) bool {
+func (e *Enforcer) HasPermissionForUser(user string, permission ...string) (bool, error) {
 	return e.HasPolicy(util.JoinSlice(user, permission...))
 }
 
@@ -211,29 +229,53 @@ func (e *Enforcer) HasPermissionForUser(user string, permission ...string) bool
 // GetRolesForUser("alice") can only get: ["role:admin"].
 // But GetImplicitRolesForUser("alice") will get: ["role:admin", "role:user"].
 func (e *Enforcer) GetImplicitRolesForUser(name string, domain ...string) ([]string, error) {
-	res := []string{}
+	var res []string
 
-	for _, rm := range e.rmMap {
+	for v := range e.rmMap {
+		roles, err := e.GetNamedImplicitRolesForUser(v, name, domain...)
+		if err != nil {
+			return nil, err
+		}
+		res = append(res, roles...)
+	}
 
-		roleSet := make(map[string]bool)
-		roleSet[name] = true
-		q := make([]string, 0)
-		q = append(q, name)
+	return res, nil
+}
 
-		for len(q) > 0 {
-			name := q[0]
-			q = q[1:]
+// GetNamedImplicitRolesForUser gets implicit roles that a user has by named role definition.
+// Compared to GetImplicitRolesForUser(), this function retrieves indirect roles besides direct roles.
+// For example:
+// g, alice, role:admin
+// g, role:admin, role:user
+// g2, alice, role:admin2
+//
+// GetImplicitRolesForUser("alice") can only get: ["role:admin", "role:user"].
+// But GetNamedImplicitRolesForUser("g2", "alice") will get: ["role:admin2"].
+func (e *Enforcer) GetNamedImplicitRolesForUser(ptype string, name string, domain ...string) ([]string, error) {
+	var res []string
 
-			roles, err := rm.GetRoles(name, domain...)
-			if err != nil {
-				return nil, err
-			}
-			for _, r := range roles {
-				if _, ok := roleSet[r]; !ok {
-					res = append(res, r)
-					q = append(q, r)
-					roleSet[r] = true
-				}
+	rm := e.GetNamedRoleManager(ptype)
+	if rm == nil {
+		return nil, fmt.Errorf("role manager %s is not initialized", ptype)
+	}
+	roleSet := make(map[string]bool)
+	roleSet[name] = true
+	q := make([]string, 0)
+	q = append(q, name)
+
+	for len(q) > 0 {
+		name := q[0]
+		q = q[1:]
+
+		roles, err := rm.GetRoles(name, domain...)
+		if err != nil {
+			return nil, err
+		}
+		for _, r := range roles {
+			if _, ok := roleSet[r]; !ok {
+				res = append(res, r)
+				q = append(q, r)
+				roleSet[r] = true
 			}
 		}
 	}
@@ -246,7 +288,6 @@ func (e *Enforcer) GetImplicitUsersForRole(name string, domain ...string) ([]str
 	res := []string{}
 
 	for _, rm := range e.rmMap {
-
 		roleSet := make(map[string]bool)
 		roleSet[name] = true
 		q := make([]string, 0)
@@ -283,7 +324,7 @@ func (e *Enforcer) GetImplicitUsersForRole(name string, domain ...string) ([]str
 // GetPermissionsForUser("alice") can only get: [["alice", "data2", "read"]].
 // But GetImplicitPermissionsForUser("alice") will get: [["admin", "data1", "read"], ["alice", "data2", "read"]].
 func (e *Enforcer) GetImplicitPermissionsForUser(user string, domain ...string) ([][]string, error) {
-	return e.GetNamedImplicitPermissionsForUser("p", user, domain...)
+	return e.GetNamedImplicitPermissionsForUser("p", "g", user, domain...)
 }
 
 // GetNamedImplicitPermissionsForUser gets implicit permissions for a user or role by named policy.
@@ -294,31 +335,47 @@ func (e *Enforcer) GetImplicitPermissionsForUser(user string, domain ...string)
 // g, alice, admin
 //
 // GetImplicitPermissionsForUser("alice") can only get: [["admin", "data1", "read"]], whose policy is default policy "p"
-// But you can specify the named policy "p2" to get: [["admin", "create"]] by    GetNamedImplicitPermissionsForUser("p2","alice")
-func (e *Enforcer) GetNamedImplicitPermissionsForUser(ptype string, user string, domain ...string) ([][]string, error) {
+// But you can specify the named policy "p2" to get: [["admin", "create"]] by    GetNamedImplicitPermissionsForUser("p2","alice").
+func (e *Enforcer) GetNamedImplicitPermissionsForUser(ptype string, gtype string, user string, domain ...string) ([][]string, error) {
 	permission := make([][]string, 0)
-	rm := e.GetRoleManager()
-	domainIndex, _ := e.GetFieldIndex(ptype, constant.DomainIndex)
+	rm := e.GetNamedRoleManager(gtype)
+	if rm == nil {
+		return nil, fmt.Errorf("role manager %s is not initialized", gtype)
+	}
+
+	roles, err := e.GetNamedImplicitRolesForUser(gtype, user, domain...)
+	if err != nil {
+		return nil, err
+	}
+	policyRoles := make(map[string]struct{}, len(roles)+1)
+	policyRoles[user] = struct{}{}
+	for _, r := range roles {
+		policyRoles[r] = struct{}{}
+	}
+
+	domainIndex, err := e.GetFieldIndex(ptype, constant.DomainIndex)
 	for _, rule := range e.model["p"][ptype].Policy {
 		if len(domain) == 0 {
-			matched, _ := rm.HasLink(user, rule[0])
-			if matched {
+			if _, ok := policyRoles[rule[0]]; ok {
 				permission = append(permission, deepCopyPolicy(rule))
 			}
-		} else if len(domain) > 1 {
+			continue
+		}
+		if len(domain) > 1 {
 			return nil, errors.ErrDomainParameter
-		} else {
-			d := domain[0]
-			matched := rm.Match(d, rule[domainIndex])
-			if !matched {
-				continue
-			}
-			matched, _ = rm.HasLink(user, rule[0], d)
-			if matched {
-				newRule := deepCopyPolicy(rule)
-				newRule[domainIndex] = d
-				permission = append(permission, newRule)
-			}
+		}
+		if err != nil {
+			return nil, err
+		}
+		d := domain[0]
+		matched := rm.Match(d, rule[domainIndex])
+		if !matched {
+			continue
+		}
+		if _, ok := policyRoles[rule[0]]; ok {
+			newRule := deepCopyPolicy(rule)
+			newRule[domainIndex] = d
+			permission = append(permission, newRule)
 		}
 	}
 	return permission, nil
@@ -333,9 +390,18 @@ func (e *Enforcer) GetNamedImplicitPermissionsForUser(ptype string, user string,
 // GetImplicitUsersForPermission("data1", "read") will get: ["alice", "bob"].
 // Note: only users will be returned, roles (2nd arg in "g") will be excluded.
 func (e *Enforcer) GetImplicitUsersForPermission(permission ...string) ([]string, error) {
-	pSubjects := e.GetAllSubjects()
-	gInherit := e.model.GetValuesForFieldInPolicyAllTypes("g", 1)
-	gSubjects := e.model.GetValuesForFieldInPolicyAllTypes("g", 0)
+	pSubjects, err := e.GetAllSubjects()
+	if err != nil {
+		return nil, err
+	}
+	gInherit, err := e.model.GetValuesForFieldInPolicyAllTypes("g", 1)
+	if err != nil {
+		return nil, err
+	}
+	gSubjects, err := e.model.GetValuesForFieldInPolicyAllTypes("g", 0)
+	if err != nil {
+		return nil, err
+	}
 
 	subjects := append(pSubjects, gSubjects...)
 	util.ArrayRemoveDuplicates(&subjects)
@@ -358,7 +424,7 @@ func (e *Enforcer) GetImplicitUsersForPermission(permission ...string) ([]string
 	return res, nil
 }
 
-// GetDomainsForUser gets all domains
+// GetDomainsForUser gets all domains.
 func (e *Enforcer) GetDomainsForUser(user string) ([]string, error) {
 	var domains []string
 	for _, rm := range e.rmMap {
@@ -371,7 +437,7 @@ func (e *Enforcer) GetDomainsForUser(user string) ([]string, error) {
 	return domains, nil
 }
 
-// GetImplicitResourcesForUser returns all policies that user obtaining in domain
+// GetImplicitResourcesForUser returns all policies that user obtaining in domain.
 func (e *Enforcer) GetImplicitResourcesForUser(user string, domain ...string) ([][]string, error) {
 	permissions, err := e.GetImplicitPermissionsForUser(user, domain...)
 	if err != nil {
@@ -410,7 +476,7 @@ func (e *Enforcer) GetImplicitResourcesForUser(user string, domain ...string) ([
 	return res, nil
 }
 
-// deepCopyPolicy returns a deepcopy version of the policy to prevent changing policies through returned slice
+// deepCopyPolicy returns a deepcopy version of the policy to prevent changing policies through returned slice.
 func deepCopyPolicy(src []string) []string {
 	newRule := make([]string, len(src))
 	copy(newRule, src)
@@ -483,9 +549,16 @@ func (e *Enforcer) GetImplicitUsersForResource(resource string) ([][]string, err
 	subjectIndex, _ := e.GetFieldIndex("p", "sub")
 	objectIndex, _ := e.GetFieldIndex("p", "obj")
 	rm := e.GetRoleManager()
+	if rm == nil {
+		return nil, fmt.Errorf("role manager is not initialized")
+	}
 
 	isRole := make(map[string]bool)
-	for _, role := range e.GetAllRoles() {
+	roles, err := e.GetAllRoles()
+	if err != nil {
+		return nil, err
+	}
+	for _, role := range roles {
 		isRole[role] = true
 	}
 
@@ -518,18 +591,25 @@ func (e *Enforcer) GetImplicitUsersForResource(resource string) ([][]string, err
 }
 
 // GetImplicitUsersForResourceByDomain return implicit user based on resource and domain.
-// Compared to GetImplicitUsersForResource, domain is supported
+// Compared to GetImplicitUsersForResource, domain is supported.
 func (e *Enforcer) GetImplicitUsersForResourceByDomain(resource string, domain string) ([][]string, error) {
 	permissions := make([][]string, 0)
 	subjectIndex, _ := e.GetFieldIndex("p", "sub")
 	objectIndex, _ := e.GetFieldIndex("p", "obj")
 	domIndex, _ := e.GetFieldIndex("p", "dom")
 	rm := e.GetRoleManager()
+	if rm == nil {
+		return nil, fmt.Errorf("role manager is not initialized")
+	}
 
 	isRole := make(map[string]bool)
 
-	for _, role := range e.GetAllRolesByDomain(domain) {
-		isRole[role] = true
+	if roles, err := e.GetAllRolesByDomain(domain); err != nil {
+		return nil, err
+	} else {
+		for _, role := range roles {
+			isRole[role] = true
+		}
 	}
 
 	for _, rule := range e.model["p"]["p"].Policy {
