@@ -21,7 +21,7 @@ import (
 	"github.com/casbin/casbin/v2/util"
 )
 
-// testGetUsersInDomain: Add by Gordon
+// testGetUsersInDomain: Add by Gordon.
 func testGetUsersInDomain(t *testing.T, e *Enforcer, name string, domain string, res []string) {
 	t.Helper()
 	myRes := e.GetUsersForRoleInDomain(name, domain)
@@ -52,7 +52,7 @@ func TestGetImplicitRolesForDomainUser(t *testing.T) {
 	testGetImplicitRolesInDomain(t, e, "alice", "domain1", []string{"role:global_admin", "role:reader", "role:writer"})
 }
 
-// TestUserAPIWithDomains: Add by Gordon
+// TestUserAPIWithDomains: Add by Gordon.
 func TestUserAPIWithDomains(t *testing.T) {
 	e, _ := NewEnforcer("examples/rbac_with_domains_model.conf", "examples/rbac_with_domains_policy.csv")
 
@@ -164,7 +164,6 @@ func TestRoleAPIWithDomains(t *testing.T) {
 
 	testGetRoles(t, e, []string{}, "non_exist", "domain2")
 	testGetRolesInDomain(t, e, "non_exist", "domain2", []string{})
-
 }
 
 func testGetPermissionsInDomain(t *testing.T, e *Enforcer, name string, domain string, res [][]string) {
@@ -212,8 +211,9 @@ func TestGetDomainsForUser(t *testing.T) {
 }
 
 func testGetAllUsersByDomain(t *testing.T, e *Enforcer, domain string, expected []string) {
-	if !util.SetEquals(e.GetAllUsersByDomain(domain), expected) {
-		t.Errorf("users in %s: %v, supposed to be %v\n", domain, e.GetAllUsersByDomain(domain), expected)
+	users, _ := e.GetAllUsersByDomain(domain)
+	if !util.SetEquals(users, expected) {
+		t.Errorf("users in %s: %v, supposed to be %v\n", domain, users, expected)
 	}
 }
 
@@ -228,11 +228,20 @@ func testDeleteAllUsersByDomain(t *testing.T, domain string, expectedPolicy, exp
 	e, _ := NewEnforcer("examples/rbac_with_domains_model.conf", "examples/rbac_with_domains_policy.csv")
 
 	_, _ = e.DeleteAllUsersByDomain(domain)
-	if !util.Array2DEquals(e.GetPolicy(), expectedPolicy) {
-		t.Errorf("policy in %s: %v, supposed to be %v\n", domain, e.GetPolicy(), expectedPolicy)
+	policy, err := e.GetPolicy()
+	if err != nil {
+		t.Error(err)
 	}
-	if !util.Array2DEquals(e.GetGroupingPolicy(), expectedGroupingPolicy) {
-		t.Errorf("grouping policy in %s: %v, supposed to be %v\n", domain, e.GetGroupingPolicy(), expectedGroupingPolicy)
+	if !util.Array2DEquals(policy, expectedPolicy) {
+		t.Errorf("policy in %s: %v, supposed to be %v\n", domain, policy, expectedPolicy)
+	}
+
+	policies, err := e.GetGroupingPolicy()
+	if err != nil {
+		t.Error(err)
+	}
+	if !util.Array2DEquals(policies, expectedGroupingPolicy) {
+		t.Errorf("grouping policy in %s: %v, supposed to be %v\n", domain, policies, expectedGroupingPolicy)
 	}
 }
 
@@ -251,7 +260,7 @@ func TestDeleteAllUsersByDomain(t *testing.T) {
 	})
 }
 
-// testGetAllDomains tests GetAllDomains()
+// testGetAllDomains tests GetAllDomains().
 func testGetAllDomains(t *testing.T, e *Enforcer, res []string) {
 	t.Helper()
 	myRes, _ := e.GetAllDomains()
@@ -269,8 +278,9 @@ func TestGetAllDomains(t *testing.T) {
 }
 
 func testGetAllRolesByDomain(t *testing.T, e *Enforcer, domain string, expected []string) {
-	if !util.SetEquals(e.GetAllRolesByDomain(domain), expected) {
-		t.Errorf("roles in %s: %v, supposed to be %v\n", domain, e.GetAllRolesByDomain(domain), expected)
+	roles, _ := e.GetAllRolesByDomain(domain)
+	if !util.SetEquals(roles, expected) {
+		t.Errorf("roles in %s: %v, supposed to be %v\n", domain, roles, expected)
 	}
 }
 
@@ -285,5 +295,4 @@ func TestGetAllRolesByDomain(t *testing.T) {
 	testGetAllRolesByDomain(t, e, "domain1", []string{"admin"})
 	testGetAllRolesByDomain(t, e, "domain2", []string{"admin"})
 	testGetAllRolesByDomain(t, e, "domain3", []string{"user"})
-
 }
