@@ -569,6 +569,75 @@ func TestGlobMatch(t *testing.T) {
 	testGlobMatch(t, "/prefix/subprefix/foobar", "*/foo", false)
 	testGlobMatch(t, "/prefix/subprefix/foobar", "*/foo*", false)
 	testGlobMatch(t, "/prefix/subprefix/foobar", "*/foo/*", false)
+
+	testGlobMatch(t, "/foo", "**/foo", true)
+	testGlobMatch(t, "/foo", "**/foo**", true)
+	testGlobMatch(t, "/foo", "**/foo/**", true)
+	testGlobMatch(t, "/foo/bar", "**/foo", false)
+	testGlobMatch(t, "/foo/bar", "**/foo**", false)
+	testGlobMatch(t, "/foo/bar", "**/foo/**", true)
+	testGlobMatch(t, "/foobar", "**/foo", false)
+	testGlobMatch(t, "/foobar", "**/foo**", true)
+	testGlobMatch(t, "/foobar", "**/foo/**", false)
+
+	testGlobMatch(t, "/prefix/foo", "**/foo", true)
+	testGlobMatch(t, "/prefix/foo", "**/foo**", true)
+	testGlobMatch(t, "/prefix/foo", "**/foo/**", true)
+	testGlobMatch(t, "/prefix/foo/bar", "**/foo", false)
+	testGlobMatch(t, "/prefix/foo/bar", "**/foo**", false)
+	testGlobMatch(t, "/prefix/foo/bar", "**/foo/**", true)
+	testGlobMatch(t, "/prefix/foobar", "**/foo", false)
+	testGlobMatch(t, "/prefix/foobar", "**/foo**", true)
+	testGlobMatch(t, "/prefix/foobar", "**/foo/**", false)
+
+	testGlobMatch(t, "/prefix/subprefix/foo", "**/foo", true)
+	testGlobMatch(t, "/prefix/subprefix/foo", "**/foo**", true)
+	testGlobMatch(t, "/prefix/subprefix/foo", "**/foo/**", true)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "**/foo", false)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "**/foo**", false)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "**/foo/**", true)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "**/foo", false)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "**/foo**", true)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "**/foo/**", false)
+
+	testGlobMatch(t, "/foo", "*/foo**", true)
+	testGlobMatch(t, "/foo", "**/foo*", true)
+	testGlobMatch(t, "/foo", "*/foo/**", true)
+	testGlobMatch(t, "/foo", "**/foo/*", false)
+	testGlobMatch(t, "/foo/bar", "*/foo**", false)
+	testGlobMatch(t, "/foo/bar", "**/foo*", false)
+	testGlobMatch(t, "/foo/bar", "*/foo/**", true)
+	testGlobMatch(t, "/foo/bar", "**/foo/*", true)
+	testGlobMatch(t, "/foobar", "*/foo**", true)
+	testGlobMatch(t, "/foobar", "**/foo*", true)
+	testGlobMatch(t, "/foobar", "*/foo/**", false)
+	testGlobMatch(t, "/foobar", "**/foo/*", false)
+
+	testGlobMatch(t, "/prefix/foo", "*/foo**", false)
+	testGlobMatch(t, "/prefix/foo", "**/foo*", true)
+	testGlobMatch(t, "/prefix/foo", "*/foo/**", false)
+	testGlobMatch(t, "/prefix/foo", "**/foo/*", false)
+	testGlobMatch(t, "/prefix/foo/bar", "*/foo**", false)
+	testGlobMatch(t, "/prefix/foo/bar", "**/foo*", false)
+	testGlobMatch(t, "/prefix/foo/bar", "*/foo/**", false)
+	testGlobMatch(t, "/prefix/foo/bar", "**/foo/*", true)
+	testGlobMatch(t, "/prefix/foobar", "*/foo**", false)
+	testGlobMatch(t, "/prefix/foobar", "**/foo*", true)
+	testGlobMatch(t, "/prefix/foobar", "*/foo/**", false)
+	testGlobMatch(t, "/prefix/foobar", "**/foo/*", false)
+
+	testGlobMatch(t, "/prefix/subprefix/foo", "*/foo**", false)
+	testGlobMatch(t, "/prefix/subprefix/foo", "**/foo*", true)
+	testGlobMatch(t, "/prefix/subprefix/foo", "*/foo/**", false)
+	testGlobMatch(t, "/prefix/subprefix/foo", "**/foo/*", false)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "*/foo**", false)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "**/foo*", false)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "*/foo/**", false)
+	testGlobMatch(t, "/prefix/subprefix/foo/bar", "**/foo/*", true)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "*/foo**", false)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "**/foo*", true)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "*/foo/**", false)
+	testGlobMatch(t, "/prefix/subprefix/foobar", "**/foo/*", false)
 }
 
 func testTimeMatch(t *testing.T, startTime string, endTime string, res bool) {
